/***
   NAME
     cr.c
   NOTES
     The problem specific definition file for the size-structured
     consumer-resource model with pulsed reproduction possibly every
     spring, if accumulated fat is sufficient and individuals are
     are mature.
 
     Terminology:
     
		X:	Irreversible mass or bone
		Y:	Reversible mass or fat
		M:	Mass or standard mass, (1+QJ)*X
		W:	Weight or total weight: X+Y

    HISTORY
    AMdR - Sep 07, 2008: Last Revision.
***/

#include  "escbox.h"


/*
 *===========================================================================
 *
 *		LABELLING ENVIRONMENT AND I-STATE VARIABLES
 *
 *===========================================================================
 */
#define	time		(env[0])
#define	food		(env[1])

#define	age		(i_state(0))
#define	bone		(i_state(1))
#define	fat		(i_state(2))

#define	IDmass		(i_const( 0))
#define	IDweight	(i_const( 1))
#define	IDarate		(i_const( 2))
#define	IDhtime		(i_const( 3))
#define	IDmaint		(i_const( 4))
#define	IDingest	(i_const( 5))
#define	IDnet_energy	(i_const( 6))
#define	IDkappa		(i_const( 7))
#define	IDmort		(i_const( 8))
#define	IDfatratio	(i_const( 9))
#define	IDfecundity	(i_const(10))
#define	IDnbegin	(i_const(11))

#define	CONS		0

#define	SMALL		0.001
#define	TINY		1.0E-6
#define MINFOOD		1.0E-12



/*
 *===========================================================================
 *
 *		DEFINING AND LABELLING CONSTANTS AND PARAMETERS
 *
 *===========================================================================
 */
#define MU_BACK		parameter[ 0]	// Background mortality rate
#define MU_J		parameter[ 1]	// Size-dep. mortality constant
#define X_MU		parameter[ 2]	// Size-dep. mortality exponent

#define AR_CON		parameter[ 3]	// Attack rate constant
#define AR_EXP		parameter[ 4]	// Attack rate exponent
#define AR_WOPT		parameter[ 5]	// Optimal consumption weight

#define DIGT_CON	parameter[ 6]	// Digestion time constant
#define DIGT_EXP	parameter[ 7]	// Digestion time exponent

#define MR_CON		parameter[ 8]	// Maintenance allometric constant
#define MR_EXP		parameter[ 9]	// Maintenance allometric exponent

#define CONS_IW		parameter[10]	// Weight of neonate individual
#define XMAT		parameter[11]	// Irreversible mass at maturation

#define QJ		parameter[12]	// Juvenile and adult reversible to
#define QA		parameter[13]	// irreversible mass ratio
#define CONVEFF		parameter[14]	// Food conversion efficiency

#define QS		parameter[15]	// Starvation mortality parameters
#define MU_S		parameter[16]	

#define R		parameter[17]	// Resource growth rate
#define K		parameter[18]	// Pelagic carrying capacity (g/m3)

#define TOTVOL		parameter[19]	// Volume of entire lake
#define SEASON		parameter[20]	// Season duration

#define	YEAR		SEASON

#define CONS_MAX_YEARS	25.0



/*
 *===========================================================================
 *
 *		DEFINING GLOBAL VARIABLES AND USER-IMPLEMENTED FUNCTIONS
 *
 *===========================================================================
 */

static void		UpdateIDcards(double *env, population *pop, int setfec);

static double		YOYsurvival, SizeOYO, MeanFecundity;
static int		ReproCohorts;



/*
 *===========================================================================
 *
 * USER INITIALIZATION ROUTINE ALLOWS OPERATIONS ON INITIAL POPULATIONS
 *
 *===========================================================================
 */

void	UserInit(int argc, char	**argv, double *env, population *pop)

{
  switch (argc)
    {
    case 5:
      X_MU = atof(argv[4]);
    case 4:
      MU_J = atof(argv[3]);
    case 3:
      MU_BACK = atof(argv[2]);
    default:
      break;
    }
fprintf(stderr, "1"); fflush(stderr);
  UpdateIDcards(env, pop, 0);

  ReportNote("\n    %s%s",
#if (BIFURCATION == 1)
	     "Bifurcation of the size-structured consumer-resource model",
#else
	     "Dynamics of the size-structured consumer-resource model",
#endif // BIFURCATION
	     "\n");

  return;
}




/*
 *===========================================================================
 *
 *	SPECIFICATION OF THE NUMBER AND VALUES OF BOUNDARY POINTS
 *
 *===========================================================================
 */

void	SetBpointNo(double *env, population *pop, int *bpoint_no)

{
  bpoint_no[CONS] = 0;

  return;
}




/*===========================================================================*/

void	SetBpoints(double *env, population *pop, population *bpoints)

{

  return;
}




/*
 *===========================================================================
 *
 *			SPECIFICATION OF DERIVATIVES
 *
 *===========================================================================
 */

void	Gradient(double *env,	  population *pop,     population *ofs,
		 double *envgrad, population *popgrad, population *ofsgrad,
		 population *bpoints)

{
  register int		i;
  double		kappa = 0.0,  net_energy = 0.0;
  double		ingest = 0.0, mort = 0.0;
  double		tot_grazing = 0.0;

  UpdateIDcards(env, pop, 0);

  for (i=0; i<cohort_no[CONS]; i++)
    {
      popgrad[CONS][i][number] =  0.0;
      popgrad[CONS][i][age]    =  1.0;
      popgrad[CONS][i][bone]   =  0.0;
      popgrad[CONS][i][fat]    = -1.0;
      if (pop[CONS][i][fat] < TINY*pop[CONS][i][bone]) continue;

      mort	 = popIDcard[CONS][i][IDmort];
      kappa	 = popIDcard[CONS][i][IDkappa];
      net_energy = popIDcard[CONS][i][IDnet_energy];
      ingest	 = popIDcard[CONS][i][IDingest];

      popgrad[CONS][i][number] = -mort*pop[CONS][i][number];
      popgrad[CONS][i][bone]   = kappa*net_energy;
      popgrad[CONS][i][fat]    = (1-kappa)*net_energy;

      tot_grazing += ingest*pop[CONS][i][number];
    }

  envgrad[0] = 1.0;
  envgrad[1] = R*(K-food) - tot_grazing/TOTVOL;

  return;
}




/*
 *===========================================================================
 *
 *		SPECIFICATION OF BETWEEN COHORT CYCLE DYNAMICS
 *
 *===========================================================================
 */

void	InstantDynamics(double *env, population *pop, population *ofs)

{
  register int		i, newcoh;
  double		juveniles = 0.0, TotalRepro = 0.0, dummy;


  if (food < MINFOOD) food = MINFOOD;		// Check negative food

  for (i=0; i<cohort_no[CONS]; i++)
    {
      // remove cohorts with no fat
      if (pop[CONS][i][fat] < TINY*pop[CONS][i][bone])
	pop[CONS][i][number] = 0.0;

      // remove very old cohorts
      if (pop[CONS][i][age] > (CONS_MAX_YEARS*YEAR))
	pop[CONS][i][number] = 0.0;

      if ((pop[CONS][i][age] < YEAR + SMALL) && (popIDcard[CONS][i][IDnbegin] > TINY))
	YOYsurvival = pop[CONS][i][number]/popIDcard[CONS][i][IDnbegin];
    }

  dummy = fmod(time, YEAR);
  if ((dummy < SMALL) || (dummy > (YEAR-SMALL)))
    {
      UpdateIDcards(env, pop, 1);

      ReproCohorts = 0;
      TotalRepro   = 0.0;
      juveniles	   = 0.0;

      for (i=0; i<cohort_no[CONS]; i++)
	{
	  if ((pop[CONS][i][fat] < TINY*pop[CONS][i][bone]) || 
	      (pop[CONS][i][number] <= 0.0)) continue;

	  if (pop[CONS][i][age] < YEAR+SMALL) SizeOYO = pop[CONS][i][bone];

	  if (pop[CONS][i][bone] > XMAT)
	    {
	      juveniles += popIDcard[CONS][i][IDfecundity]*pop[CONS][i][number];
	      pop[CONS][i][fat] = QJ*pop[CONS][i][bone];

	      ReproCohorts++;
	      TotalRepro += pop[CONS][i][number];
	    }
	}

      if (juveniles)
	{
	  newcoh = AddCohorts(pop, CONS, 1);
	  pop[CONS][newcoh][number] = juveniles;
	  pop[CONS][newcoh][age]    = 0.0;
	  pop[CONS][newcoh][bone]   = 1/(1+QJ)*CONS_IW;
	  pop[CONS][newcoh][fat]    = (QJ)/(1+QJ)*CONS_IW;

	  popIDcard[CONS][newcoh][IDnbegin]    = juveniles;
	  popIDcard[CONS][newcoh][IDfecundity] = 0.0;

	  SetStepSize(SMALL);
	}

      if (TotalRepro) MeanFecundity = juveniles/TotalRepro;
      else MeanFecundity = 0.0;
    }

  UpdateIDcards(env, pop, 0);

  return;
}




/*
 *===========================================================================
 *
 *			SPECIFICATION OF OUTPUT VARIABLES
 *
 *===========================================================================
 */

void	DefineOutput(double *env, population *pop, double *output)

{
  register int		i;
  double		num, wght, biom;

  /*
   * OUTPUT VARIABLES:
   * [ 0]	 2: Food density
   * [ 1]	 3: YOY
   * [ 2]	 4: YOY biomass
   * [ 3]	 5: 1+
   * [ 4]	 6: 1+ biomass
   * [ 5]	 7: Juvenile 1+
   * [ 6]	 8: Juvenile 1+ biomass
   * [ 7]	 9: Adult 1+
   * [ 8]	10: Adult 1+  biomass
   * [ 9]	11: YOY survival
   * [10]	12: OYO body size
   * [11]	13: # of reproducing cohorts
   * [12]	14: Mean fecundity
   */

  UpdateIDcards(env, pop, 0);

#if (BIFURCATION == 1)
  LabelState(CONS, "Bif. parameter = %.4f   T = %.0f", parameter[BifParIndex], env[0]);
#endif /* BIFURCATION */

  output[0] = food;
  for (i=0; i<cohort_no[CONS]; i++)
    {
      num  = pop[CONS][i][number];
      wght = pop[CONS][i][bone]+pop[CONS][i][fat];
      biom = wght*pop[CONS][i][number];
      
      if (pop[CONS][i][age] < YEAR - SMALL)
	{
	  output[ 1] += num;
	  output[ 2] += biom;
	}
      else
	{
	  output[ 3] += num;
	  output[ 4] += biom;

	  if (pop[CONS][i][bone] < XMAT)
	    {
	      output[ 5] += num;
	      output[ 6] += biom;
	    }
	  else
	    {
	      output[ 7] += num;
	      output[ 8] += biom;
	    }
	}
    }

  output[ 9] = YOYsurvival;
  output[10] = SizeOYO;
  output[11] = ReproCohorts;
  output[12] = MeanFecundity;

  return;
}



/*==============================================================================*/

static void	UpdateIDcards(double *env, population *pop, int setfec)

{
  register int		i;
  double		mass = 0.0, weight = 0.0;
  double		arate = 0.0, htime = 0.0, ingest = 0.0, maint = 0.0;
  double		q = 0.0, kappa = 0.0, net_energy = 0.0;
  double		mort = 0.0, fecundity  = 0.0;

  for (i=0; i<cohort_no[CONS]; i++)
    {
      if (pop[CONS][i][fat] < TINY*pop[CONS][i][bone]) continue;

      mass   = pop[CONS][i][bone]*(1 + QJ);
      weight = pop[CONS][i][bone] + pop[CONS][i][fat];

      arate  = AR_CON*pow(mass/AR_WOPT*exp(1-mass/AR_WOPT), AR_EXP);
      htime  = DIGT_CON*pow(mass, DIGT_EXP);
      maint  = MR_CON*pow(weight, MR_EXP);

      ingest = arate*food/(1+arate*htime*food);
      net_energy = (CONVEFF*ingest - maint);

      if (pop[CONS][i][bone] < XMAT) q = QJ;
      else q = QA;
      if (net_energy > 0.0)
	{
	  if (q*pop[CONS][i][bone] - pop[CONS][i][fat] < 0.0)
	    kappa = 1/(q+1);
	  else
	    kappa = pop[CONS][i][fat]/((1+q)*q*pop[CONS][i][bone]);
	}
      else
	kappa = 0.0;

      if (X_MU > TINY)
	mort = MU_BACK + MU_J*exp(-pop[CONS][i][bone]/X_MU);
      else
	{
	  if (pop[CONS][i][bone] < XMAT)
	    mort  = (MU_J+MU_BACK);
	  else
	    mort  = MU_BACK;
	}

      if (pop[CONS][i][fat] < QS*pop[CONS][i][bone])
	mort += MU_S*(QS*pop[CONS][i][bone]/pop[CONS][i][fat] - 1.0);

      if ((pop[CONS][i][bone] > XMAT) &&
	  (pop[CONS][i][fat]  > QJ*pop[CONS][i][bone]))
	fecundity = (pop[CONS][i][fat]-QJ*pop[CONS][i][bone])/(2.0*CONS_IW);
      else
	fecundity = 0.0;

      popIDcard[CONS][i][IDmass]	= mass;
      popIDcard[CONS][i][IDweight]	= weight;
      popIDcard[CONS][i][IDarate]	= arate;
      popIDcard[CONS][i][IDhtime]	= htime;
      popIDcard[CONS][i][IDmaint]	= maint;
      popIDcard[CONS][i][IDingest]	= ingest;
      popIDcard[CONS][i][IDnet_energy]	= net_energy;
      popIDcard[CONS][i][IDkappa]	= kappa;
      popIDcard[CONS][i][IDmort]	= mort;
      popIDcard[CONS][i][IDfatratio]	= pop[CONS][i][fat]/pop[CONS][i][bone];

      if (setfec)
	popIDcard[CONS][i][IDfecundity]	= fecundity;
    }

  return;
}



/*==============================================================================*/
